@@ -4,37 +4,34 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef HAVE_UNIBILIUM
-# include "unibilium.h"
-#else
-# include <curses.h>
-# include <term.h>
-
-/* term.h has defined 'lines' as a macro. Eugh. We'd really rather prefer it
- * didn't pollute our namespace so we'll provide some functions here and then
- * #undef the name pollution
- */
-static inline int terminfo_lines(void)   { return lines; }
-static inline int terminfo_columns(void) { return columns; }
-
-# undef lines
-# undef columns
-#endif
+#define strneq(a,b,n) (strncmp(a,b,n)==0)
 
 struct XTermDriver {
   TickitTermDriver driver;
 
+  int dcs_offset;
+  char dcs_buffer[16];
+
   struct {
     unsigned int altscreen:1;
     unsigned int cursorvis:1;
     unsigned int cursorblink:1;
+    unsigned int cursorshape:2;
     unsigned int mouse:1;
     unsigned int keypad:1;
   } mode;
 
   struct {
+    unsigned int cursorshape:1;
     unsigned int slrm:1;
   } cap;
+
+  struct {
+    unsigned int cursorvis:1;
+    unsigned int cursorblink:1;
+    unsigned int cursorshape:2;
+    unsigned int slrm:1;
+  } initialised;
 };
 
 static void print(TickitTermDriver *ttd, const char *str)
@@ -186,8 +183,7 @@ static void erasech(TickitTermDriver *ttd, int count, int moveend)
   }
 }
 
-/* clear() may collide with something from curses.h or term.h */
-static void ttd_clear(TickitTermDriver *ttd)
+static void clear(TickitTermDriver *ttd)
 {
   tickit_termdrv_write_strf(ttd, "\e[2J", 4);
 }
@@ -308,6 +304,10 @@ static int getctl_int(TickitTermDriver *ttd, TickitTermCtl ctl, int *value)
       *value = xd->mode.mouse;
       return 1;
 
+    case TICKIT_TERMCTL_CURSORSHAPE:
+      *value = xd->mode.cursorshape;
+      return 1;
+
     case TICKIT_TERMCTL_KEYPAD_APP:
       *value = xd->mode.keypad;
       return 1;
@@ -343,9 +343,9 @@ static int setctl_int(TickitTermDriver *ttd, TickitTermCtl ctl, int value)
       return 1;
 
     case TICKIT_TERMCTL_CURSORBLINK:
-      /* We don't actually know whether this was enabled initially, so best
-       * just to always apply this
-       */
+      if(xd->initialised.cursorblink && !xd->mode.cursorblink == !value)
+        return 1;
+
       tickit_termdrv_write_str(ttd, value ? "\e[?12h" : "\e[?12l", 0);
       xd->mode.cursorblink = !!value;
       return 1;
@@ -359,7 +359,12 @@ static int setctl_int(TickitTermDriver *ttd, TickitTermCtl ctl, int value)
       return 1;
 
     case TICKIT_TERMCTL_CURSORSHAPE:
-      tickit_termdrv_write_strf(ttd, "\e[%d q", value * 2 + (xd->mode.cursorblink ? -1 : 0));
+      if(xd->initialised.cursorshape && xd->mode.cursorshape == value)
+        return 1;
+
+      if(xd->cap.cursorshape)
+        tickit_termdrv_write_strf(ttd, "\e[%d q", value * 2 + (xd->mode.cursorblink ? -1 : 0));
+      xd->mode.cursorshape = value;
       return 1;
 
     case TICKIT_TERMCTL_KEYPAD_APP:
@@ -401,24 +406,103 @@ static void start(TickitTermDriver *ttd)
 
   // Find out if DECSLRM is actually supported
   tickit_termdrv_write_strf(ttd, "\e[?69$p");
+
+  // Also query the current cursor visibility, blink status, and shape
+  tickit_termdrv_write_strf(ttd, "\e[?25$p\e[?12$p\eP$q q\e\\");
 }
 
-static void gotkey(TickitTermDriver *ttd, TermKey *tk, const TermKeyKey *key)
+static int started(TickitTermDriver *ttd)
+{
+  struct XTermDriver *xd = (struct XTermDriver *)ttd;
+
+  return xd->initialised.cursorvis &&
+         xd->initialised.cursorblink &&
+         xd->initialised.cursorshape &&
+         xd->initialised.slrm;
+}
+
+static void gotkey_modereport(struct XTermDriver *xd, int initial, int mode, int value)
+{
+  if(initial == '?') // DEC mode
+    switch(mode) {
+      case 12: // Cursor blink
+        if(value == 1)
+          xd->mode.cursorblink = 1;
+        xd->initialised.cursorblink = 1;
+        break;
+      case 25: // DECTCEM == Cursor visibility
+        if(value == 1)
+          xd->mode.cursorvis = 1;
+        xd->initialised.cursorvis = 1;
+        break;
+      case 69: // DECVSSM
+        if(value == 1 || value == 2)
+          xd->cap.slrm = 1;
+        xd->initialised.slrm = 1;
+        break;
+    }
+}
+
+static void gotkey_decrqss(struct XTermDriver *xd, char status, char *args, size_t arglen)
+{
+  if(strneq(args + arglen - 2, " q", 2)) { // DECSCUSR
+    int value;
+    if(status == '1' && sscanf(args, "%d", &value)) {
+      // value==1 or 2 => shape == 1, 3 or 4 => 2, etc..
+      int shape = (value+1) / 2;
+      xd->mode.cursorshape = shape;
+      xd->cap.cursorshape = 1;
+    }
+    xd->initialised.cursorshape = 1;
+  }
+}
+
+static int gotkey(TickitTermDriver *ttd, TermKey *tk, const TermKeyKey *key)
 {
   struct XTermDriver *xd = (struct XTermDriver *)ttd;
 
   if(key->type == TERMKEY_TYPE_MODEREPORT) {
     int initial, mode, value;
     termkey_interpret_modereport(tk, key, &initial, &mode, &value);
+    gotkey_modereport(xd, initial, mode, value);
 
-    if(initial == '?') // DEC mode
-      switch(mode) {
-        case 69: // DECVSSM
-          if(value == 1 || value == 2)
-            xd->cap.slrm = 1;
-          break;
-      }
+    return 1;
   }
+  // TODO: Long term we'll move libtermkey's code into terminal drivers and
+  // stop using it. Until then we'll have to have our own DCS parser
+  else if(key->type == TERMKEY_TYPE_UNICODE &&
+          key->modifiers == TERMKEY_KEYMOD_ALT &&
+          key->code.codepoint == 'P') {
+    xd->dcs_offset = 0;
+    return 1;
+  }
+  else if(xd->dcs_offset != -1 &&
+          key->type == TERMKEY_TYPE_UNICODE &&
+          key->modifiers == 0) {
+    if(xd->dcs_offset < sizeof xd->dcs_buffer)
+      xd->dcs_buffer[xd->dcs_offset++] = key->utf8[0]; // TODO: UTF-8 in DCS?
+    return 1;
+  }
+  else if(key->type == TERMKEY_TYPE_UNICODE &&
+          key->modifiers == TERMKEY_KEYMOD_ALT &&
+          key->code.codepoint == '\\') {
+    if(xd->dcs_offset == -1)
+      return 1;
+
+    size_t cmdlen = 1;
+    while(cmdlen < xd->dcs_offset &&
+          xd->dcs_buffer[cmdlen-1] < 0x40)
+      cmdlen++;
+
+    if(cmdlen == 3 && strneq(xd->dcs_buffer + 1, "$r", 2))
+      gotkey_decrqss(xd, xd->dcs_buffer[0], xd->dcs_buffer + cmdlen, xd->dcs_offset - cmdlen);
+
+    xd->dcs_offset = -1;
+
+    return 1;
+  }
+
+  return 0;
 }
 
 static void stop(TickitTermDriver *ttd)
@@ -445,13 +529,14 @@ static void destroy(TickitTermDriver *ttd)
 static TickitTermDriverVTable xterm_vtable = {
   .destroy    = destroy,
   .start      = start,
+  .started    = started,
   .stop       = stop,
   .print      = print,
   .goto_abs   = goto_abs,
   .move_rel   = move_rel,
   .scrollrect = scrollrect,
   .erasech    = erasech,
-  .clear      = ttd_clear,
+  .clear      = clear,
   .chpen      = chpen,
   .getctl_int = getctl_int,
   .setctl_int = setctl_int,
@@ -475,32 +560,14 @@ static TickitTermDriver *new(TickitTerm *tt, const char *termtype)
   xd->driver.vtable = &xterm_vtable;
   xd->driver.tt = tt;
 
-  xd->mode.altscreen = 0;
+  xd->dcs_offset = -1;
+
+  memset(&xd->mode, 0, sizeof xd->mode);
   xd->mode.cursorvis = 1;
-  xd->mode.mouse     = 0;
 
-  /* This will be set to 1 later if the terminal responds appropriately to the
-   * DECRQM on DECVSSM
-   */
-  xd->cap.slrm = 0;
+  memset(&xd->cap, 0, sizeof xd->cap);
 
-#ifdef HAVE_UNIBILIUM
-  {
-    unibi_term *ut = unibi_from_term(termtype);
-    if(ut) {
-      tickit_term_set_size(tt, unibi_get_num(ut, unibi_lines), unibi_get_num(ut, unibi_columns));
-
-      unibi_destroy(ut);
-    }
-  }
-#else
-  {
-    int err;
-    if(setupterm((char*)termtype, 1, &err) == OK) {
-      tickit_term_set_size(tt, terminfo_lines(), terminfo_columns());
-    }
-  }
-#endif
+  memset(&xd->initialised, 0, sizeof xd->initialised);
 
   return (TickitTermDriver*)xd;
 }
